#!/bin/sh
#################################################################
#
#  File Name:  rfxinstall.sh
#
#  Module Abstract:
#   This module contains the Bourne shell script which installs
#   the rest of the RasterFLEX software release under Solaris 2.
#
#################################################################

#################################################################
# Copyright (c) 1998 Visicom
#################################################################

#################################################################
# List of packages 
#################################################################
ItemList="\
    VITrdrvr \
    VITropwin \
    VITrfddx \
    VITrxserv \
    VITrxfont \
    VITrxsupt \
    VITrxman \
    "

#################################################################
# Package Names
#################################################################
VITrdrvrName="RasterFLEX Device Driver and Utilities"
VITropwinName="RasterFLEX OpenWindows Environment"
VITrfddxName="RasterFLEX Loadable DDX Environment"
VITrxservName="RasterFLEX X11R5 Server"
VITrxfontName="RasterFLEX X11R5 Font Files"
VITrxsuptName="RasterFLEX X11R5 Development Environment"
VITrxmanName="RasterFLEX X11R5 Manual Pages"

#################################################################
# Get the directory into which install/config tools and
# source packages are located.
#################################################################
if [ $# -lt 1 ]
then
    InstallDir="/cdrom/Solaris2"
else
    InstallDir="$1"
fi

#################################################################
# The default installation directory 
#################################################################
DefaultDir="/opt/VITrflex"

#################################################################
# Check to see if we are running as the super user.
#################################################################

touch /tmp/sutest$$
chown root /tmp/sutest$$ > /dev/null 2>&1
if [ "$?" != '0' ]
then
    echo ""
    echo "You must be the super user to install the RasterFLEX software."
    echo ""
    rm -f /tmp/sutest$$
    exit 1
fi
rm -f /tmp/sutest$$

#################################################################
# The current version.
#################################################################

Version='5.0'

#################################################################
# The year.
#################################################################

Year=`date +19%y`

#################################################################
#
# Echo Argument*
#
#   Argument    An argument to print.
#
# Action
#
#   Echos the given arguments to the controlling terminal without
#   appending a newline.
#
# Return Value
#
#   None
#
#################################################################

Echo()
{
echo -n "$*" > /dev/tty
}

#################################################################
#
# EchoLn Argument*
#
#   Argument    An argument to print.
#
# Action
#
#   Echos the given arguments to the controlling terminal with a
#   newline appended.
#
# Return Value
#
#   None
#
#################################################################

EchoLn()
{
echo "$*" > /dev/tty
}

#################################################################
#
# Increment variable
#
#   variable    The variable to increment.
#
# Action
#
#   Increments the variable.
#
# Return Value
#
#   None.
#
#################################################################

Increment()
{
_Increment_variable="$1"

eval _Increment_value=\$$_Increment_variable
eval $_Increment_variable=`expr $_Increment_value '+' '1'`

}

#################################################################
#
# Decrement variable
#
#   variable    The variable to decrement.
#
# Action
#
#   Decrements the variable.
#
# Return Value
#
#   None.
#
#################################################################

Decrement()
{
_Decrement_variable="$1"

eval _Decrement_value=\$$_Decrement_variable
eval $_Decrement_variable=`expr $_Decrement_value '-' '1'`

}

#################################################################
#
# DirectoryName Pathname
#
#   Pathname    A pathname of interest.
#
# Action
#
#   Extracts the directory portion of the given pathname.
#
# Return Value
#
#   The directory portion of Pathname.
#
#################################################################

DirectoryName()
{
DN_Pathname="$1"

DN_Scratch=`expr \
  ${DN_Pathname-.}'/' : '\(/\)[^/]*/$' \
  \| ${DN_Pathname-.}'/' : '\(.*[^/]\)//*[^/][^/]*//*$' \
  \| .`

echo "$DN_Scratch"
}

#################################################################
#
# MakeDirectory Directory
#
#   Directory   The directory to make.
#
# Action
#
#   Makes the given directory including any superior directories
#   which may not exist.
#
# Return Value
#
#   None.
#
#################################################################

MakeDirectory()
{
MD_Directory="$1"

MD_SP='0'
eval MD_Stack_$MD_SP=$MD_Directory
RecursiveMakeDirectory
}

RecursiveMakeDirectory()
{
eval RMD_DirectoryToMake=\$MD_Stack_$MD_SP

if [ ! -d "$RMD_DirectoryToMake" ]
then

    RMD_Parent=`DirectoryName $RMD_DirectoryToMake`

    if [ ! -d "$RMD_Parent" ]
    then
        MD_SP=`expr $MD_SP + 1`
        eval MD_Stack_$MD_SP="$RMD_Parent"
        RecursiveMakeDirectory
        MD_SP=`expr $MD_SP - 1`
    fi

    eval mkdir \$MD_Stack_$MD_SP >/dev/null 2>&1

fi

}

#################################################################
#
# FormatText
#
# Action
#
# Formats text using nroff
#
# Return Value
#
#   None
#
#################################################################
FormatText()
{
EchoLn ""
nroff <<END_OF_FILE | sed -n -e '/^END_OF_FILE/q;p'
.nh
.fi
.pl 24
.ll 70
$1
END_OF_FILE
}

#################################################################
#
# SIGINTHandler
#
# Action
#
#   Receives and processes an interrupt.
#
# Return Value
#
#   None.
#
#################################################################

SIGINT_handler()
{
if [ -f /tmp/RFXDEST ]
then
    rm -f /tmp/RFXDEST
fi

EchoLn ""
EchoLn "********************************************"
EchoLn "* RasterFLEX Software Installation Aborted *" 
EchoLn "********************************************"
EchoLn ""
exit 1
}

trap 'SIGINT_handler' 2

#################################################################
# MAIN
#################################################################

#
# Loadable DDX Modules are not supported for Solaris 2.0, 2.1, and 2.2
# In this case, offer X11/NeWS as an option and not Loadable DDX
#
#
# RFX does not support openwin Xnews 3.x any more
# Or more clearly, we don't support Solaris 2.[012]
#
case `uname -r` in

    5.[012]) 
	LoadableDDX="NO"
	ItemList=`echo $ItemList | sed "s/VITrfddx//"`
	ItemList=`echo $ItemList | sed "s/VITropwin//"`
	;;
    *) 
	LoadableDDX="YES"
	ItemList=`echo $ItemList | sed "s/VITropwin//"`
	;;

esac

while :
do

    #################################################################
    # Allow selection of a specific configuration
    #################################################################
    for Item in $ItemList
    do
        eval  ${Item}Load=""
        eval  ${Item}Dir="$DefaultDir"
    done
    FinalLoadList=""

    FormatText "\
Please select the desired software configuration which you would like to
load or select \"custom\" to perform selection of individual packages
from a menu.
.br
END_OF_FILE
"

    Full="Full Install Entire Release"
    OW="OpenWindows Install OpenWindows Configuration"
    X11R5="X11R5 Install X11 Release 5 Configuration"
    Custom="Custom Install Custom Configuration"

    Config=`ckitem -n -p "Select desired software configuration:" "$Full" "$OW" "$X11R5" "$Custom"` || exit $?
    if [ $Config = $Full ]
    then

        for Item in $ItemList
        do
            eval  ${Item}Load="ON"
        done

    elif [ $Config = $OW ]
    then

        VITrdrvrLoad="ON"
 	if [ "$LoadableDDX" = "YES" ]
	then
       	    VITrfddxLoad="ON"
#	else
#            VITropwinLoad="ON"
	fi

    elif [ $Config = $X11R5 ]
    then

        VITrdrvrLoad="ON"
        VITrxservLoad="ON"
        VITrxfontLoad="ON"
        VITrxsuptLoad="ON"
        VITrxmanLoad="ON"
    else

        #################################################################
        # Explain the release selection menu.
        #################################################################

        FormatText "\
The RasterFLEX software release consists of several distinct packages.  The
following menu lists these packages.  To request installation of a package,
enter the number to the left of the desired package.  An asterisk (*) will
appear next to all items which have been selected.
To remove a selected item from the list, simply enter the number for the
item again.
.sp
When you have completed selection of all items, enter \"done\" to continue
with the software installation.
.br
END_OF_FILE
"

        ExitFlag="False"
        while [ $ExitFlag = False ]
        do

            #################################################################
            # Build the Menu List 
            #################################################################
            if [ -f /tmp/VITPKGMENU ]
            then
                rm -f /tmp/VITPKGMENU
            fi
            touch /tmp/VITPKGMENU
            for Item in $ItemList
            do
                eval Scratch=\$${Item}Load
                eval Name=\$${Item}Name
                if [ -z "$Scratch" ]
                then  
                    echo "$Item $Name" >> /tmp/VITPKGMENU
                else
                    echo "$Item(*) $Name" >> /tmp/VITPKGMENU
                fi 
            done

            #################################################################
            # Prompt for selection 
            #################################################################
            Sel=`ckitem -i done -n -m 100 -p "Select the desired packages (\"done\" to continue)" -f /tmp/VITPKGMENU` || exit $?
            rm -f /tmp/VITPKGMENU


            #################################################################
            # Process selection keywords 
            #################################################################
            for SelItem in $Sel
            do
                if [ $SelItem = done ]
                then
                    ExitFlag="True"
                    continue
                fi
                for ChkItem in $ItemList
                do 
                    Found=`expr $SelItem : "\($ChkItem\).*"`
                    if [ -n "$Found" ] 
                    then
                        eval Scratch=\$${Found}Load
                        if [ -z "$Scratch" ]
                        then
                            eval ${Found}Load="ON" 
                        else
                            eval ${Found}Load="" 
                        fi
                        break
                    fi
                done
            done
        done
    fi

    #################################################################
    # Compute the final list of archives to load.
    #################################################################

    NumberOfItemsToLoad='0'

    for Item in $ItemList
    do
        eval Scratch=\$${Item}Load
        if [ "$Scratch" ]
        then
            FinalLoadList="$FinalLoadList $Item"
            Increment NumberOfItemsToLoad
        fi
    done

    #################################################################
    # Check to see if the user selected anything at all.
    #################################################################

    if [ "$NumberOfItemsToLoad" = "0" ]
    then
        FormatText "\
You have not selected any packages to load.  RasterFLEX software installation
has been aborted. 
.br
END_OF_FILE
"
        exit 0
    fi

    ##################################################################
    # Allow selection of the manner in which destination directories
    # are selected.
    ##################################################################

    FormatText "\
Now that you have selected which items you want to load, you must choose
directories in which to load them.  By default, all packages in the 
RasterFLEX release are stored beneath the directory /opt/VITrflex.
.sp
You may select the default installation, select a different top-level
directory under which to store the entire release, or select the
destination directories on a package-by-package basis.
.br
END_OF_FILE
"
    Location=`ckitem -n -p "Selection installation type:" \
 		"Default All packages stored under /opt/VITrflex" \
		"Select All packages stored under a selected directory" \
		"Custom Select directories on a package-by-package basis"` || exit $?

    #################################################################
    # Explain destination directories and find out if the user wants
    # to load everything under a single directory.
    #################################################################
    if [ $Location = Select ]
    then

        #############################################################
        # The user wants to load everything under a single directory.
        # Find out which one.
        #############################################################
        MainDirectory=""
        while [ -z "$MainDirectory" ] 
        do
            MainDirectory=`ckpath -y -p "Under what directory should the release be loaded?"` || exit $?

            if [ ! -d $MainDirectory ]
            then
FormatText "\
The directory $MainDirectory does not exist.
.br
END_OF_FILE
"
                ans=`ckyorn -d y -p "Would you like to create the directory $MainDirectory? <y>"` || exit $?
                if [ $ans = y -o $ans = yes ]
                then
                    MakeDirectory $MainDirectory
                    if [ ! -d $MainDirectory ]
                    then
                        FormatText "\
The directory $MainDirectory could not be created.  Please select a different
destination directory for the RasterFLEX software release.
.br
END_OF_FILE
"
                        MainDirectory=""
                    fi
                    chmod 755 $MainDirectory
                else
                    MainDirectory=""
                fi
            else

                if [ ! -w $MainDirectory ]
                then
                    FormatText "\
The directory $MainDirectory can not be written to.  Perhaps it is on a NFS
mounted file system to which you cannot write as root from this system. 
Please select another destination directory for the RasterFLEX software 
release.
.br
END_OF_FILE
"
                    MainDirectory=""
                fi
            fi

        done

        #############################################################
        # Set the destination directory of each item. Then, make sure
        # that directory exists.
        #############################################################
        for Item in $FinalLoadList
        do
            eval ${Item}Dir=$MainDirectory
        done

    else

        if [ $Location = Custom ]
        then

            #############################################################
            # The user wants to select individual destination
            # directories. Explain how this is going to work.
            #############################################################

            FormatText "\
The following menu lists the packages which you have selected.  For each 
package, the current installation directory is listed.  To change an item,
simply select the item and you will be prompted to enter the new destination
directory.
.sp
When you have completed selection of the destination directories, type \"done\"
to continue with software installation.
.br
END_OF_FILE
"

            #############################################################
            # Assist the user in changing the destination directories.
            #############################################################
            ExitFlag="False"
            while [ $ExitFlag = False ]
            do
                if [ -f /tmp/VITPKGMENU ]
                then
                    rm -f /tmp/VITPKGMENU
                fi
                touch /tmp/VITPKGMENU
                for Item in $ItemList
                do
                    eval Load=\$${Item}Load
                    if [ -n "$Load" ]
                    then
                        eval Dest=\$${Item}Dir
                        eval Name=\$${Item}Name
                        echo "$Item->$Dest	$Name" >> /tmp/VITPKGMENU
                    fi
                done

                #############################################################
                # Prompt for selection
                #############################################################
                Sel=`ckitem -i done -n -p "Select the package to change (\"done\" to continue)" -f /tmp/VITPKGMENU` || exit $?
                rm -f /tmp/VITPKGMENU

                #############################################################
                # Process selection keywords
                #############################################################
                for SelItem in $Sel
                do
                    if [ $SelItem = done ]
                    then
                        ExitFlag="True"
                        continue
                    fi
                    for ChkItem in $ItemList
                    do
                        Found=`expr $SelItem : "\($ChkItem\).*"`
                        if [ -n "$Found" ]
                        then
                            MainDirectory=""
                            while [ -z "$MainDirectory" ]
                            do
                                MainDirectory=`ckpath -y -p "Under what directory should $Found be loaded?"` || exit $?

                                if [ ! -d $MainDirectory ]
                                then
                    FormatText "\
The directory $MainDirectory does not exist.
.br
END_OF_FILE
"
                                    ans=`ckyorn -d y -p "Would you like to create the directory $MainDirectory? <y>"` || exit $?
                                    if [ $ans = y -o $ans = yes ]
                                    then
                                        MakeDirectory $MainDirectory
                                        if [ ! -d $MainDirectory ]
                                        then
                                            FormatText "\
The directory $MainDirectory could not be created.  Please select a different
destination directory for the package $Found.
.br
END_OF_FILE
"
                                            MainDirectory=""
                                        fi
                                        chmod 755 $MainDirectory
                                    else
                                        MainDirectory=""
                                    fi
                                else

                                    if [ ! -w $MainDirectory ]
                                    then
                                        FormatText "\
The directory $MainDirectory can not be written to.  Perhaps it is on a NFS
mounted file system to which you cannot write as root from this system.
Please select another destination directory for the package $Found.
.br
END_OF_FILE
"
                                        MainDirectory=""
                                    fi
                                fi
                            done
                            eval ${Found}Dir=$MainDirectory
                            break;
                        fi
                    done
                done
            done
        fi
    fi

    #################################################################
    # Summarize.
    #################################################################
    FormatText "\
The following list summarizes the packages which you have selected for
installation.  If the selections are correct, enter \"y\" below to 
proceed with software installation.  If you would like to respecify
your selections, enter \"n\" to reselect packages/destination directories.
.sp
.br
END_OF_FILE
"

    for Item in $ItemList
    do
        eval Scratch=\$${Item}Load
	if [ "$Scratch" ] 
        then
            eval Dest=\$${Item}Dir
            eval Name=\$${Item}Name
            EchoLn "$Item ($Name) -> $Dest"
        fi 
    done

    #################################################################
    # Give the user one last change to respecify
    #################################################################
    ans=`ckyorn -d y -p "Is this satisfactory? <y>"` || exit $? 
    if [ $ans = y -o $ans = yes ]
    then
        break
    fi

done

#########################################################
# Store the destination of the packages in the file
# /tmp/RFXDEST.  All packages contain request scripts
# which use the value in this file to set the base
# directory for package installation.
#########################################################
if [ -f /tmp/RFXDEST ]
then
    rm -f /tmp/RFXDEST
fi
touch /tmp/RFXDEST

#################################################################
# Load the release.
#################################################################
InstallList=
for Item in $ItemList
do

    #############################################################
    # Check if we're supposed to load this item.
    #############################################################

    eval Scratch=\$${Item}Load

    if [ "$Scratch" ]
    then

        eval Dest=\$${Item}Dir
   
        #########################################################
        # Add the package to the installation list
        #########################################################
        InstallList="$InstallList $Item"

        #########################################################
        # Write the <package> <dest> pair to /tmp/RFXDEST
        #########################################################
        echo "${Item} ${Dest}" >> /tmp/RFXDEST
   fi

done

#########################################################
# Load the items from the CD-ROM 
#########################################################
FormatText "\
The packages which you have requested will now be loaded from the software
release CD-ROM using the pkgadd (1) utility.
.sp
.br
END_OF_FILE
"

#########################################################
# Get the items off the CD-ROM.
#########################################################
for Item in $InstallList
do
    eval BASEDIR=\$${Item}Dir
    export BASEDIR
    pkgadd -a $InstallDir/rfxadmin -d $InstallDir $Item
done

#if [ -f /tmp/RFXDEST ]
#then
#    rm -f /tmp/RFXDEST
#fi

#################################################################
# Invoke the Mechanic if desired
#################################################################
if [ -n "`pkginfo | grep VITrdrvr`" ]
then
    DriverDir=`pkgparam VITrdrvr RFXTOP`
    FormatText "\
A companion utility, rfxconfig.sh, can generate a sample .cshrc
file which contains the appropriate environment variable assigments
and aliases based upon the manner in which you have chosen to install
the RasterFLEX software release.
.sp
.br
END_OF_FILE
"
    ans=`ckyorn -d y -p "Would you like me to invoke the configuration utility? <y>"` || exit $?
    if [ $ans = y -o $ans = yes ]
    then 
        cd $DriverDir
        exec $DriverDir/rfxconfig.sh
    else
        FormatText "\
The utility is a shell script named \"rfxconfig.sh\" in the
directory \"$DriverDir\".  You can run it at any time.
.br
END_OF_FILE
"
    fi
fi

#################################################################
# Exit.
#################################################################
exit 0
